from .common import NetlabError

CODES = []


def _add(cls):
    CODES.append(cls)
    return cls


@_add
class ContentNotFoundError(NetlabError):
    """con_id is not found."""
    str_error = "E_CONTENT_NOT_FOUND"


@_add
class ExerciseNotFoundError(NetlabError):
    """ex_id is not found."""
    str_error = "E_EXERCISE_NOT_FOUND"
