NL_OK = "OK"
NL_PASSED = "PASSED"
NL_ERROR = "ERROR"
HDR_SEVERITY_LEVELS = {
    'FATAL': 1,
    'ERROR': 2,
    'WARN': 3,
    'SUCCESS': 4,
    'INFO': 5,
    'TRACE0': 6,
    'TRACE1': 7,
    'TRACE2': 8
}
